"""
The :mod:`gccov.scatter` plot scatter bubbles.
"""

# Author: Jie Li <mm.jlli6t@gmail.com>
# License: GNU v3.0
# Copyrigth: 2019

import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
from biosut import io_seq
import os
import pandas as pd

class scatter:
	def __init__(self, data, outpdf, bins_dir=None, suffix='fa', scale=False, size=1):
#	def __init__(self, data, outpdf, **targs):
		self.data = data
		self.bins_dir = bins_dir
		self.suffix = suffix
		self.outpdf = outpdf
		self.scale = scale
		self.size = 10000/size

	def plot(self):
		if self.bins_dir:
			full_contigs_color = self._color_set()
		else:
			full_contigs_color = ['grey'] * len(self.data)
		print('You are plotting with color: ', set(full_contigs_color))
#		self.data.color = full_contigs_color
#		self.data.srt_values(by='color')
		plt.figure(1)
		if self.scale:
#			self.data.plot.scatter(x='GC_content', y='Coverage',
#								c=full_contigs_color, linewidths=.5,
#								edgecolors='k', alpha=.5,
#								s=self.data['Seq_length']/10000)
			self.data.plot.scatter(x='gc_ratio', y='coverage',
								c='white', linewidth=.5,
								edgecolors=full_contigs_color,
								alpha=.5, s=self.data['seq_length']/self.size)
		else:
			self.data.plot.scatter(x='gc_ratio', y='coverage',
								c=full_contigs_color, alpha=.5, s=1)

		plt.xlabel('GC%')
		plt.ylabel('Coverage')
		plt.savefig(self.outpdf)

	def _color_set(self):
		flag = 0
		color_sets = ['blue', 'red', 'yellow', 'green', 'orange', 'purple', 'pink']
		contigs_color = {}
		for f in os.listdir(self.bins_dir):
			if f.endswith(self.suffix):
				#print(f)
				print(color_sets[flag])
				f_ids = io_seq.seq_to_dict(os.path.join(self.bins_dir, f), outqual=False)
				for i in f_ids.keys():
					f_ids[i] = color_sets[flag]
				contigs_color.update(f_ids)
				flag += 1
		# then assign grey to all unbinned contigs
		full_contigs_color=[contigs_color[i] if i in contigs_color.keys() else 'grey' for i in self.data.index]
		return full_contigs_color




"""
The :mod:`gccov.coverm` pile up coverm.
"""

# Author: Jie Li <mm.jlli6t@gmail.com>
# License: GNU v3.0
# Copyrigth: 2019

import os
import sys
from biosut import gt_exe, gt_file

class coverm:
	def __init__(self, bam, outfile):
		gt_exe.is_executable('coverm')
		self.bam = bam
		self.outfile = outfile

	def run(self):
		cmd = 'coverm contig --bam-files %s --methods mean --min-covered-fraction 0 --output-format dense --contig-end-exclusion 0 > %s' % (self.bam, self.outfile)
		gt_exe.exe_cmd(cmd, shell=True)
		gt_file.check_file_exist(self.outfile, check_empty=True)



"""
The :mod:`gccov.main` pileup the workflow.
"""

# Author: Jie Li <mm.jlli6t@gmail.com>
# License: GNU v3.0
# Copyrigth: 2019

import os
import sys
import argparse

import pandas as pd
from biosut.gt_path import real_path

sys.path = [os.path.join((os.path.dirname(real_path(__file__))), '..')] + sys.path

from biosut import gt_file, gt_path
from biosut.io_seq import gc_to_dict

from gccov.scatter import scatter
from gccov.coverm import coverm
from gccov.version import Version

def read_arg(args):

	p = argparse.ArgumentParser(description=Version.show_version())
	required_argument = p.add_argument_group('Required arguments')

	required_argument.add_argument('--contigs', required=True,
								help='contigs/scaffolds for GC content')

	mutual_required_argument = p.add_argument_group('Mutually exclusive required argument')
	mutual_rgs = mutual_required_argument.add_mutually_exclusive_group(required=True)
	mutual_rgs.add_argument('--coverage', help='coverage file, with column name Coverage')
	mutual_rgs.add_argument('--bam_file', help='sorted bam file')

	optional_arguments = p.add_argument_group('Optional arguments')
	optional_arguments.add_argument('-scale', default=False, action='store_true',
					help='set to scale scatter dots with your scaffolds/contigs length')
	optional_arguments.add_argument('-size', default=1, type=float,
					help='bubles relative size you want, default is 1, you can set to 1.5, 3, 5 or so')
	optional_arguments.add_argument('-prefix', default='gc_coverage',
					help='prefix of outputs, [gc_coverage]')
	optional_arguments.add_argument('-contig_len', default=2500, type=float,
					help='contig length cutoff for GC content and plot, [2500]')
	optional_arguments.add_argument('-cov_width', default='0', type=str,
					help='cov range you want to plot, for example 0-100, single 0 means all, [0]')
	optional_arguments.add_argument('-bins_dir', default=None,
					help='bins dir to color genomes you provide')
	optional_arguments.add_argument('-suffix', default='fa',
					help='suffix of bins if you profile -bins_dir, [fa]')
	optional_arguments.add_argument('-outdir', default=os.getcwd(),
					help='output dir')
	return p.parse_args()

class stream:

	def exe(args):
		arg = read_arg(args)
		outdir = gt_path.sure_path_exist(arg.outdir)
		gc_table = gc_to_dict(arg.contigs, arg.contig_len, length=True)

		gc_table = pd.DataFrame.from_dict(gc_table).T
		gc_table.columns = ['gc_count', 'seq_length']
		gc_table['gc_ratio'] = gc_table.gc_count/gc_table.seq_length*100.
		print("Finished get GC content.\n")

		gc_table.to_csv(outdir +'/'+ arg.prefix + '_gc_content.txt', sep='\t')

		if arg.bam_file:
			cov = os.path.join(outdir, arg.prefix+'.coverage')
			coverm_pile = coverm(arg.bam_file, cov)
			coverm_pile.run()
		else:
			cov = arg.coverage
		gt_file.check_file_exist(cov, check_empty=True)

		cov = pd.read_csv(cov, sep="\t", header=0, index_col=0)
		print(cov)
		cov.columns = ['coverage']
		print('Finished get Coverage.\n')

			# get contigs have both gc and coverage
		new = cov.merge(gc_table, how='inner', left_index=True, right_index=True)
		new.to_csv(outdir + '/' + arg.prefix +'_gc_and_coverage.csv', sep='\t')

		if '-' in arg.cov_width:
			cov_width = [float(i) for i in arg.cov_width.split('-')]
			new = new[(new.coverage >= cov_width[0]) & (new.coverage <= cov_width[1])]

		scatter_plot = scatter(new, outdir+'/'+arg.prefix+'.pdf', \
								arg.bins_dir, arg.suffix, arg.scale, \
								arg.size)
#								scatter_plot = scatter(new, outdir+pars['prefix']+'.pdf', **pars)
		scatter_plot.plot()



# -*- coding:utf-8 -*-

from setuptools import setup, find_packages
import os
from gccov.version import Version

setup(
	name = 'gccov',
	description = 'Visualize gc & coverage.',
	version = Version.get_version(),
	url = 'https://github.com/jlli6t/gccov',
	author = 'Jie Li',
	author_email = 'mm.jlli6t@gmail.com',
	maintainer = 'Jie Li',
	maintainer_email = 'mm.jlli6t@gmail.com',

	classifiers = [
				'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
				'Programming Language :: Python :: 3 :: Only',
				'Operating System :: Unix',
		],
	keywords = 'biology bioinformatics',
	scripts=['bin/gccov'],
	packages = find_packages(),
	include_package_data=True,
	python_requires = '>=3.6',
	install_requires=['biosut>=2.0.0',],
)



"""
The :mod:`gccov.version` printing version information.
"""

# Author: Jie Li <mm.jlli6t@gmail.com>
# License: GNU v3.0
# Copyrigth: 2018

import os
from biosut.gt_path import abs_path

class Version:

    def get_version():
        ver1 = os.path.dirname(abs_path(__file__)) + '/version'
        ver2 = os.path.dirname(abs_path(__file__)) + '/../version'
        if os.path.isfile(ver1):return open(ver1).readline().strip()
        if os.path.isfile(ver2):return open(ver2).readline().strip()
        return 'Unknow version'

    @classmethod
    def show_version(cls):
        ver = cls.get_version()
        name = os.path.split(os.path.dirname(__file__))[1]
        print('\n\n\t\t%s version * %s *\n\n' % (name, ver))
